<?php

/**
 * @file
 * Contains Drupal\tvi\Controller\TaxonomyViewsIntegratorTermPageController.
 */

namespace Drupal\tvi\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\taxonomy\TermInterface;
use Drupal\views\Views;
use Drupal\Core\Config;
use Drupal\taxonomy\TermStorage;

class TaxonomyViewsIntegratorTermPageController extends ControllerBase {

  /**
   * Returns a renderable array from a View based on the taxonomy view integrator configuration settings for the requested term entity.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function render(TermInterface $taxonomy_term) {
    return $this->getTaxonomyTermView($taxonomy_term);
  }

  /**
   * Get the taxonomy view integrator settings for this term entity.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return \Drupal\Core\Config\Config
   */
  protected function getTermConfigSettings(TermInterface $taxonomy_term) {
    return $this->config('tvi.taxonomy_term.' . $taxonomy_term->id());
  }

  /**
   * Get the taxonomy view integrator settings for this terms vocabulary entity.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return \Drupal\Core\Config\Config
   */
  protected function getVocabularyConfigSettings(TermInterface $taxonomy_term) {
    return $this->config('tvi.taxonomy_vocabulary.' . $taxonomy_term->getVocabularyId());
  }

  /**
   * Get the default taxonomy view integrator settings.
   * @return \Drupal\Core\Config\Config
   */
  protected function getDefaultConfigSettings() {
    return $this->config('tvi.global_settings');
  }

  /**
   * Return the default taxonomy term View.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return bool|null|string|void
   */
  protected function getDefaultTaxonomyTermView(TermInterface $taxonomy_term) {
    return views_embed_view('taxonomy_term', 'page_1', $taxonomy_term->id());
  }

  /**
   * Return the taxonomy term View per taxonomy view integrator settings.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return bool|null|string|void
   */
  protected function getTaxonomyTermView(TermInterface $taxonomy_term) {
    $config = $this->getTermConfigSettings($taxonomy_term);
    $enabled = $config->get('enable_override');
    $view_name = 'taxonomy_term';
    $view_id = 'page_1';
    $check_vocab = true;
    $check_global = true;

    // View return scenarios:
    //   term is TVI configured, most specific override
    //   term is a child of another term, its top most TVI enabled parent has 'children inherit' option enabled
    //   no term is enabled, but the vocabulary is tvi enabled
    //   no term tvi, no vocab tvi, but global tvi settings exist and enabled
    //   nothing enabled, return default taxonomy term view

    if ($enabled) {
      $view_name = $config->get('view');
      $view_id = $config->get('view_display');
    } else {
      // @todo the logic in this block definitely could use refactoring

      // check parent terms for settings
      foreach ($this->getTermParents($taxonomy_term) as $parent) {
        $config = $this->getTermConfigSettings($parent);
        // if parent is enabled and has 'children inherit settings' is checked off
        if ($config->get('enable_override') && $config->get('inherit_settings')) {
          $view_name = $config->get('view');
          $view_id = $config->get('view_display');
          $check_vocab = false;
          break;
        }
      }

      // check vocab for settings
      if ($check_vocab) {
        $config = $this->getVocabularyConfigSettings($taxonomy_term);

        if ($config->get('enable_override') && $config->get('inherit_settings')) {
          $view_name = $config->get('view');
          $view_id = $config->get('view_display');
        }
      }

      // check for global settings
      $config = $this->getDefaultConfigSettings();

      if ($config->get('enable_override')) {
        $view_name = $config->get('view');
        $view_id = $config->get('view_display');
      }
    }

    // if the option to pass all args to views is enabled, pass them on
    // @todo get args after {taxonomy_term} and pass them

    return views_embed_view($view_name, $view_id, $taxonomy_term->id());
  }

  /**
   * Wrapper method for obtaining parents of a given taxonomy term.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return mixed
   */
  protected function getTermParents(TermInterface $taxonomy_term) {
    return $this->entityTypeManager()->getStorage('taxonomy_term')->loadParents($taxonomy_term->id());
  }
}